-------------------------------------------------------------------
-- Date 		: 20 Mar 2011
-- Databse		: Ms SQL Server
-- Description 		: ISAMESSO Sync Data Housekeeping Maintenance Script
-- IMS Version		: 8.1.x / 8.2.x
--
--  Syntax : sp_purgeDelSyncDataOlderThan '<del mode>', <days> 
--  * <del mode> - Mandatory, text, case sensitive  
--				 - ''ABOUT'' display script information 
--				 - ''DELETE'' to perform actual removal 
--				 - ''HELP'' display script syntax and example 
--				 - ''SIMULATE'' to simulate deletion without actual removal 
--  * <days> - Mandatory, numeric  
--           - Enter 0 to delete all 
--
-- Example : 
--  * exec sp_purgeDelSyncDataOlderThan 'SIMULATE',30  - Simulate purging of deleted sync data older than 30 days without performing actual removal
--  * exec sp_purgeDelSyncDataOlderThan 'DELETE',14    - Purges deleted sync data older than 2 weeks (14 days) from database
--  * exec sp_purgeDelSyncDataOlderThan 'DELETE',0     - Purges all deleted sync data from database
--
-- user, system and machine-scoped Sync Data 
-------------------------------------------------------------------
--
-- DROP PROCEDURE sp_purgeDelSyncDataOlderThan;
--
-- EXEC AS USER = 'imsdb7907';
-- exec sp_purgeDelSyncDataOlderThan 'ABOUT',30;
-- exec sp_purgeDelSyncDataOlderThan 'SIMULATE',30;
-- exec sp_purgeDelSyncDataOlderThan 'DELETE',30;
-- exec sp_purgeDelSyncDataOlderThan 'HELP',30;
-- exec sp_purgeDelSyncDataOlderThan '',0;
--
-- Tables affected : 
--  IMSTemplatedSyncRepo 
--

CREATE PROCEDURE sp_purgeDelSyncDataOlderThan
@IN_operationCmd VARCHAR(8),
@IN_days INT
AS
BEGIN

	DECLARE @var_strSQL NVARCHAR(256) ;
	DECLARE @var_bitLoop BIT;
		
	DECLARE @var_tsRetainDate DATETIME;
	DECLARE @var_iBatchNum INT ; SET @var_iBatchNum = 10000;
	DECLARE @var_iTmpNum INT ; 
	
	DECLARE @var_info_script_ver VARCHAR(16); SET @var_info_script_ver = '1.0.0';
	DECLARE @var_info_db_ver VARCHAR(128);
	
	DECLARE @var_info_start_time DATETIME;
	DECLARE @var_info_end_time DATETIME;
	DECLARE @var_info_elapsed_time VARCHAR(32);
	
	DECLARE @var_ims_schema VARCHAR(32); 
	DECLARE @var_tableschema VARCHAR(32); 

	DECLARE @var_showSyntax BIT; SET @var_showSyntax = 0;
	DECLARE @var_invalid_schema BIT; SET @var_invalid_schema = 0;
	DECLARE @var_del_result INT ; SET @var_del_result = 0;
	
	SET @var_tsRetainDate = DATEADD(day,-@IN_days,CURRENT_TIMESTAMP);

	SET @var_ims_schema = SCHEMA_NAME();
		
	SET @var_tableschema = @var_ims_schema + '.';
	
	-- verify IMS schema
	DECLARE @var_iTemplatedSyncRepoCount INT;
	SET @var_iTemplatedSyncRepoCount = (SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = @var_ims_schema AND UPPER(TABLE_NAME) = 'IMSTEMPLATEDSYNCREPO');
	SET @var_invalid_schema = 1-@var_iTemplatedSyncRepoCount; -- 0 = OK

	IF @var_invalid_schema = 0
	BEGIN

		PRINT N'command : sp_purgeDelSyncDataOlderThan ''' + @IN_operationCmd + ''', ' + CAST(@IN_days AS VARCHAR);

		IF @IN_operationCmd IN ('DELETE','SIMULATE')
		BEGIN
			PRINT N' Input  > Delete mode : ' + @IN_operationCmd;
			PRINT N' Input  > Days : ' + CAST(@IN_days AS VARCHAR);
			PRINT N' Output > Purge records older than (YYYY.MM.DD) : ' + CONVERT(NVARCHAR, @var_tsRetainDate, 102);
			PRINT N' ';
			
			SET NOCOUNT ON;
  			-- Capture start time
			SET @var_info_start_time = CURRENT_TIMESTAMP;

			DECLARE @var_strAction VARCHAR(16);
			IF @IN_operationCmd = 'DELETE'
				SET @var_strAction = 'purged';
			ELSE
				SET @var_strAction = 'calculating';	
			

			SET @var_del_result = 0;
			IF @IN_operationCmd = 'DELETE'
			BEGIN
				SET @var_bitLoop = 1;
				SET @var_strSQL = N'DELETE TOP('+CAST(@var_iBatchNum as NVARCHAR)+') FROM '+@var_tableschema+'IMSTemplatedSyncRepo WHERE isDeleted = 1 AND updateTimeStamp < '''+CAST(@var_tsRetainDate as varchar)+'''';
				 
				WHILE @var_bitLoop=1
				BEGIN
					BEGIN TRAN
						EXECUTE(@var_strSQL);
						SET @var_iTmpNum = @@ROWCOUNT;
						SET @var_del_result = @var_del_result + @var_iTmpNum;
						IF (@var_iTmpNum < @var_iBatchNum) SET @var_bitLoop = 0;
					COMMIT TRAN;
				END;
			END ELSE
			BEGIN
				SET @var_strSQL = N'SELECT @result = COUNT(*) FROM '+@var_tableschema+'IMSTemplatedSyncRepo WHERE isDeleted = 1 AND updateTimeStamp < '''+CAST(@var_tsRetainDate as varchar)+'''';
				EXECUTE sp_executesql @var_strSQL, N'@result int OUTPUT', @result=@var_del_result OUTPUT;
			END;
			PRINT N' - '+@var_strAction+' IMSTemplatedSyncRepo ... '+CAST(@var_del_result AS VARCHAR);

			-- Capture end time
			SET @var_info_end_time = CURRENT_TIMESTAMP;
			DECLARE @var_timediff DATETIME; SET @var_timediff = @var_info_end_time-@var_info_start_time;
			SET @var_info_elapsed_time = CAST(DATEPART(day,@var_timediff)-1 AS VARCHAR)+'d ' 
				+ CAST(DATEPART(hour,@var_timediff) AS VARCHAR) +'h ' + CAST(DATEPART(minute,@var_timediff) AS VARCHAR)+'m '
				+ CAST(DATEPART(second,@var_timediff) AS VARCHAR)+'s ' + CAST(DATEPART(millisecond,@var_timediff) AS VARCHAR)+'ms ';

			PRINT N'';
			PRINT N' Start time   : ' + CONVERT(NVARCHAR, @var_info_start_time, 121);
			PRINT N' End time     : ' + CONVERT(NVARCHAR, @var_info_end_time, 121);
			PRINT N' Elapsed time : ' + @var_info_elapsed_time;
			SET NOCOUNT OFF;
		END	
		ELSE IF @IN_operationCmd = 'ABOUT'
		BEGIN
			SET @var_info_db_ver = REPLACE(REPLACE(@@VERSION,CHAR(10),' '),'	','');
			PRINT N' ISAMESSO Sync Data Housekeeping Maintenance Script';
			PRINT N'  - Database Name 	 : ' + DB_NAME();
			PRINT N'  - Schema Name 	 : ' + @var_ims_schema;
			PRINT N'  - Script Version   : ' + @var_info_script_ver;
			PRINT N'  - Database Version : ' + @var_info_db_ver;

		END ELSE 
		BEGIN	
			IF @IN_operationCmd <> 'HELP'
			BEGIN
				PRINT N' Error : Invalid delete mode '''+@IN_operationCmd+'''';
				PRINT N' ';
			END;
			SET @var_showSyntax = 1;
		END;
	END ELSE
	BEGIN
		PRINT N' Error : Invalid ISAMESSO schema '''+@var_ims_schema+'''';
		PRINT N' ';
		SET @var_showSyntax = 1;
	END;

	IF @var_showSyntax = 1
	BEGIN
		PRINT N' Syntax : sp_purgeDelSyncDataOlderThan ''<del mode>'', <days> ';
		PRINT N' * <del mode> - Mandatory, text, case sensitive';
		PRINT N'              - ''ABOUT'' display script information';
		PRINT N'              - ''DELETE'' to perform actual removal ';
		PRINT N'              - ''HELP'' display script syntax and example';
		PRINT N'              - ''SIMULATE'' to simulates purging without actual removal';
		PRINT N' * <days> - Mandatory, numeric';
		PRINT N'          - Enter 0 to delete all';
		PRINT N'';
		PRINT N'Example : ';
		PRINT N' * sp_purgeDelSyncDataOlderThan ''SIMULATE'',30 - Simulate purging of deleted sync data older than 30 days without performing actual removal'
		PRINT N' * sp_purgeDelSyncDataOlderThan ''DELETE'',14   - Purge deleted sync data older than 2 weeks (14 days) from database'
		PRINT N' * sp_purgeDelSyncDataOlderThan ''DELETE'',0    - Purge all deleted sync data from database'
		
	END;
	
END
